/*
  This file is part of GerQUtils.

  GerQUtils is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  GerQUtils is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  GerQUtils.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef GERQ_DATACONTAINER_H
#define GERQ_DATACONTAINER_H


#include "gerqutilsbase.h"

#include "dataadapter.h"
#include <QHash>
#include <QVariant>


namespace GerQ {


  typedef QHash<QString, DataObject*>::iterator DataIterator;


  class DataContainer {

    public:

      DataContainer(DataAdapter *aAdapter);
      virtual ~DataContainer();

      DataAdapter *getAdapter();
      QString getConnection();

      RefContainer *getParent();
      void setParent(RefContainer *aValue);
      void* getSelf();
      void setSelf(void* aValue);

      int getCount();
      DataIterator beginData();
      DataIterator endData();
      DataObject *getData(QVariant aID);
      void getAll(bool aAssertRead = true);

      bool read(DataObject *aData);
      void write(DataObject *aData);

      static InstContainer *addInstDefault(
        const QString &aTableName, const QString &aIDColumn,
	DataObject* (*aCreateData)(),
	void (*aRead)(DataObject*, SqlDataAdapter*),
	void (*aWrite)(DataObject*, SqlDataAdapter*),
	RefContainer *aParent = NULL,
	QString aConnection = "");
      static RefContainer *addRefDefault(
        const QString &aTableName, const QString &aIDColumn,
	void (*aRead)(DataObject*, SqlDataAdapter*),
	void (*aWrite)(DataObject*, SqlDataAdapter*),
	RefContainer *aParent = NULL,
	QString aConnection = "");
      static DataContainer *getDefaultContainer(QString &aTableName);
      static void clearDefaultContainers();

      static DataObject *getDataDefault(DataContainer *aContainer,
        SqlDataAdapter *aSql, const QString aFieldName);
      static void setDataDefault(SqlDataAdapter *aSql,
        const QString aFieldName, DataObject *aData);

      virtual QVariant getNullID();

      static QVariant getDefaultNullID();
      static void setDefaultNullID(const QVariant &aValue);

    protected:

      virtual DataObject *yetUnknownData(QVariant aID) = ABSTRACT;

      friend class DataObject;
      QHash<QString, DataObject*> fData;

    private:

      friend class DataAdapter;
      DataAdapter *fAdapter;
      RefContainer *fParent;

      void* *fSelf;

      friend class Associator;
      QHash<Associator*, Associator*> fAssociators;
      void addAssociator(Associator* aAssociator);
      void removeAssociator(Associator* aAssociator);

      QHash<QString, QString> fLoadInProgress;

      friend class DefaultInstContainer;
      friend class DefaultRefContainer;
      static void removeDefaultContainer(const QString &aTableName);

      static QHash<QString, DataContainer*> fDefaultContainers;

      static QVariant fDefaultNullID;

  }; // DataContainer


} // GerQ


#endif
