/*
  This file is part of GerQUtils.

  GerQUtils is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  GerQUtils is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  GerQUtils.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "refcontainer.h"

#include "dataobject.h"


namespace GerQ {

  RefContainer::RefContainer(DataAdapter *aAdapter): DataContainer(aAdapter) {}

  void RefContainer::addChild(DataContainer *aChild) {
    if (!fChildren.contains(aChild)) {
      fChildren.insert(aChild, aChild);
    } // if
  } // RefContainer::addChild

  ChildCtrIterator RefContainer::beginOfChildren() {
    ChildCtrIterator result(fChildren);
    return result;
  } // RefContainer::beginOfChildren

  void RefContainer::putNewData(DataObject *aData) {
    QString ID = aData -> getID().toString();
    fData.insert(ID, aData);
  } // RefContainer::putNewData

  void RefContainer::removeChild(DataContainer *aChild) {
    if (fChildren.contains(aChild)) {
      fChildren.remove(aChild);
    } // if
  } // RefContainer::removeChild

  DataObject* RefContainer::yetUnknownData(QVariant aID) {
    if (aID == getNullID()) {
      return NULL;
    } // if
    /*
      Since a RefContainer only represents an abstract anchestor class, it's
      not meant to create any data objects it doesn't know yet. So just ask all
      of your child containers, if they know the ID requested:
    */
    DataObject *result = NULL;
    ChildCtrIterator iter = beginOfChildren();
    while (iter.hasNext() && !result) {
      iter.next();
      result = iter.value() -> getData(aID);
    } // while
    return result;
  } // RefContainer::yetUnknownData

} // GerQ
